# Copyright 2019-2020 Not Just A Toy Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import enum

__all__ = (
    'ProtocolVersion',
    'PacketType',
    'QOSLevel',
    'ConnectReturnCode',
    'SubscribeReturnCode',
)


class ProtocolVersion(enum.IntEnum):
    MQTTv311 = 4


class PacketType(enum.IntEnum):
    CONNECT = 0x10
    CONNACK = 0x20
    PUBLISH = 0x30
    PUBACK = 0x40
    PUBREC = 0x50
    PUBREL = 0x60
    PUBCOMP = 0x70
    SUBSCRIBE = 0x80
    SUBACK = 0x90
    UNSUBSCRIBE = 0xA0
    UNSUBACK = 0xB0
    PINGREQ = 0xC0
    PINGRESP = 0xD0
    DISCONNECT = 0xE0


class QOSLevel(enum.IntEnum):
    QOS_0 = 0x00
    QOS_1 = 0x01
    QOS_2 = 0x02


class ConnectReturnCode(enum.IntEnum):
    CONNECTION_ACCEPTED = 0x00
    UNACCEPTABLE_PROTOCOL_VERSION = 0x01
    IDENTIFIER_REJECTED = 0x02
    SERVER_UNAVAILABLE = 0x03
    BAD_USERNAME_PASSWORD = 0x04
    NOT_AUTHORIZED = 0x05


class SubscribeReturnCode(enum.IntEnum):
    SUCCESS_MAXIMUM_QOS_0 = 0x00
    SUCCESS_MAXIMUM_QOS_1 = 0x01
    SUCCESS_MAXIMUM_QOS_2 = 0x02
    FAILURE = 0x80



# Copyright 2019-2020 Not Just A Toy Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import re

from .enums import ConnectReturnCode

__all__ = (
    'CONNECT_RETURN_CODE_DESCRIPTIONS',
    'MAXIMUM_PAYLOAD_SIZE',
    'MAXIMUM_TOPIC_LENGTH',
    'MAXIMUM_PACKET_ID',
    'TOPIC_FILTER_REGEX',
    'TOPIC_NAME_REGEX',
)


CONNECT_RETURN_CODE_DESCRIPTIONS = {
    ConnectReturnCode.CONNECTION_ACCEPTED:
        "Connection accepted",
    ConnectReturnCode.UNACCEPTABLE_PROTOCOL_VERSION:
        "The server does not support the level of the MQTT protocol requested by the client",
    ConnectReturnCode.IDENTIFIER_REJECTED:
        "The client identifier is correct UTF-8 but not allowed by the server",
    ConnectReturnCode.SERVER_UNAVAILABLE:
        "The network connection has been made but the MQTT service is unavailable",
    ConnectReturnCode.BAD_USERNAME_PASSWORD:
        "The data in the user name or password is malformed",
    ConnectReturnCode.NOT_AUTHORIZED:
        "The client is not authorized to connect"
}

MAXIMUM_PAYLOAD_SIZE: int = 268435455
MAXIMUM_TOPIC_LENGTH: int = 65535
MAXIMUM_PACKET_ID: int = 65536

TOPIC_FILTER_REGEX = re.compile(r'^/?(\+|[\w-]+)(/(\+|[\w-]+))*(/#)?$')
TOPIC_NAME_REGEX = re.compile(r'^/?[\w-]+(/[\w-]+)*$')



# Copyright 2019-2020 Not Just A Toy Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .enums import ConnectReturnCode

__all__ = (
    'Error',
    'ProtocolError',
    'AlreadyConnectedError',
    'ConnectionClosedError',
    'ServerDiedError',
    'ConnectionCloseForcedError',
    'ConnectFailedError',
    'AccessRefusedError',
    'ConnectionLostError',
)


class Error(Exception):
    pass


class ProtocolError(Error):
    pass


class AlreadyConnectedError(Error):
    pass


class ConnectionClosedError(Error):
    pass


class ServerDiedError(Error):
    pass


class ConnectionCloseForcedError(ConnectionClosedError):
    pass


class ConnectionLostError(ConnectionClosedError):
    pass


class ConnectFailedError(Error):
    pass


class AccessRefusedError(Error):

    def __init__(self, message: str, return_code: ConnectReturnCode) -> None:
        self.message = message
        self.return_code = return_code

    def __str__(self) -> str:
        return self.message
